#!/usr/bin/env bash

##############################################################################################
#                          Dataset Utils Functions                                           #
#                                                                                            #
# Functions to facilitate dataset handling related  with renku and the renku knowledge base. #
# Most functions are generalizable, but some specifically target omnibenchmark scripts and   #
# are only valid or useful in an omnibenchmark context/repository                            #
#                                                                                            #
# Change History                                                                             #
# 18/04/2021  Oksana Riba Grognuz    Original code: functions to interact with datasets and  #
#                                    query renku's knowledge base using their user endpoint. #
# 10/05/2021  Almut Lütge            Major update: Add functions to handle datasets and      #
#                                    manage dataset files.                                   #
#                                                                                            #
# authors: Oksana Riba Grognuz & Almut Lütge                                                 #
##############################################################################################
##############################################################################################
##############################################################################################

##############################################################################################

## --------------------------------------------------------------------------------------- ##
## -------------------------------- Prerequisites  ----------------------------------------##
## --------------------------------------------------------------------------------------- ##

{ # try
    source src/config.sh
} || { 
    echo "Warning, src/config.sh does not exist and may be required for some functions."
}

##############################################################################################

## ---------------------------------------------------------------------------------------- ##
## ------------------------------- Help Documentations  ------------------------------------##
## ---------------------------------------------------------------------------------------- ##


help_query_datasets_by_string()
{
   echo "Function to query the renku knowledge base endpoint for a specific string. Will return the IDs of all datasets with this string in their description, name, keyword or title."
   echo
   echo "Syntax: query_datasets_by_string STRING"
   echo
   echo "Arguments:"
   echo "STRING         String to query the knowledge base for"
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_match_datasets_keywords()
{
   echo "Function to compare a specific keyword with the dataset keyword of the input dataset and return any target dataset features, if keywords match."
   echo
   echo "Syntax: match_datasets_keywords DATASET_ID KEYWORD FIELD"
   echo
   echo "Arguments:"
   echo "DATASET_ID     Dataset ID of for the dataset to match the keyword."
   echo "KEYWORD        Word to match the datasets keyword."
   echo "Field          Field to provide the output for if keywords match."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_query_datasets_by_keyword()
{
   echo "Function to query the renku knowledge base endpoint for all datasets with a specific keyword and returns a specified output field for those that match."
   echo
   echo "Syntax: query_datasets_by_keyword KEYWORD FIELD"
   echo
   echo "Arguments:"
   echo "KEYWORD        Word to match the datasets keyword."
   echo "Field          Field to provide the output for if keywords match."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_datasets_properties_by_id()
{
   echo "Function to return a defined dataset property/field by dataset ID."
   echo
   echo "Syntax: datasets_properties_by_id DATASET_ID FIELD"
   echo
   echo "Arguments:"
   echo "DATASET_ID     Dataset ID"
   echo "Field          Field to provide the output for if keywords match."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_import_datasets_by_keyword()
{
   echo "Function to import all datasets with a specified keyword, if not already present and update those that are already present."
   echo
   echo "Syntax: import_datasets_by_keyword KEYWORD"
   echo
   echo "Arguments:"
   echo "KEYWORD        Keyword to select datasets for import/update by."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_import_datasets_by_string()
{
   echo "Function to import all datasets with a specified string in their name, description, title or keyword, if not already present and update those that are already present."
   echo
   echo "Syntax: import_datasets_by_string STRING"
   echo
   echo "Arguments:"
   echo "STRING         String to select datasets for import/update by."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


## ---------------------------------------------------------------------------------------- ##
## ----------------------- Functions to query datasets on the KG ---------------------------##
## ---------------------------------------------------------------------------------------- ##

function query_datasets_by_string () {

    # Input String to query
    # Output dataset IDs of all datasets with this string in name,title, description or keyword.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_query_datasets_by_string
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    #query
    curl -s "https://renkulab.io/knowledge-graph/datasets?query=$1" | jq -r '.[] | .identifier'
}


function match_datasets_keywords () {
    
    # Input1 Dataset ID
    # Input2 Keyword to match the datasets keyword
    # Input3 Field to return for matches (default identifier)
    # Output Return fields value, if the datasets keyword matches the queried one.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_match_datasets_keywords
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
   # match keywords
   local dataset_id=$1
   local keyword=`echo $2 | awk '{print tolower($0)}'`
   local field=${3:-identifier}
   
   curl -s "https://renkulab.io/knowledge-graph/datasets/$dataset_id" \
   | jq -r 'select(.keywords[] | ascii_downcase == ('\"$keyword\"')) | .'\"$field\"''
   }
   
   
function query_datasets_by_keyword () {
   
   # Input1 Keyword to query.
   # Input2 Dataset field tor return (Default identifier).
   # Output Return fields value of all datasets matching the queried keyword.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_query_datasets_by_keyword
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
   
   # Query
   local keyword=$1
   local field=${2:-identifier}  
   
   for dataset_id in `query_datasets_by_string "$keyword"`
   do 
       match_datasets_keywords $dataset_id "$keyword" $field
   done
   }
   

function datasets_properties_by_id () {

   # Input  Dataset ID
   # Output Return fields value of the input dataset.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_dataset_properties_by_id
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
   
   # Query
   local dataset_id=$1
   local field=${2:-name}  
   
   curl -s "https://renkulab.io/knowledge-graph/datasets/$dataset_id" | jq -r '.'\"$field\"''
   }
   
   
function import_datasets_by_keyword () {

   # Input  Keyword to query datasets for
   # Output Imports all datasets with the input keyword that are not already imported and updates those that are already imported.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_import_datasets_by_keyword
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
   
      
   # Import/update datasets
   local keyword=$1
   for dataset_id in `query_datasets_by_keyword $keyword identifier`
   do
      
      local data_name=`datasets_properties_by_id $dataset_id name`
      local data_path="data/${data_name}"
      local data_url=`datasets_properties_by_id $dataset_id url`
      
      if [ -d "${data_path}" ]
      then
         renku dataset update $data_name
      else
         renku dataset import -y $data_url
      fi
   done
   }



function import_datasets_by_string () {

   # Input  String to query datasets for
   # Output Imports all datasets with the input string in name, description, title or keyword that are not already imported and updates those that are already imported.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_import_datasets_by_string
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
   
   
   #Import/Update
   local keyword=$1
   for dataset_id in `query_datasets_by_string $keyword identifier`
   do
      
      local data_name=`datasets_properties_by_id $dataset_id name`
      local data_path="data/${data_name}"
      local data_url=`datasets_properties_by_id $dataset_id url`
      
      if [ -d "${data_path}" ]
      then
         renku dataset update $data_name
      else
         renku dataset import -y $data_url
      fi
   done
   }


   
function dataset_files_by_id () {

   local dataset_id=$1
   curl -s "https://renkulab.io/knowledge-graph/datasets/${dataset_id}" \
                       | jq -r ".hasPart | .[] | .atLocation"
   }




## ---------------------------------------------------------------------------------------- ##
## ---------------------- Functions to handle datasets using renku -------------------------##
## ---------------------------------------------------------------------------------------- ##

function create_dataset () {
    
    create_dataset_usage() { 
       echo "create_dataset: [-n <name> -t <title> -d <description> -k <keyword(s)>]" 1>&2; exit; 
       }
    
    local create_opt=("-t ${DATA_VARS[title]}" "-d ${DATA_VARS[description]}" "-k ${TAG_LIST}")
    #local create_opt=""
    local name=${DATA_VARS[name]}
    local OPTIND o name
    
    while getopts ":n:t:d:k:" o; do
        case "${o}" in
            n)
                name="${OPTARG}"
                create_opt=()
                ;;
            t)
                create_opt+=("-t ${OPTARG}")
                ;;
            d)
                create_opt+=("-d ${OPTARG}")
                ;;
            k)
                create_opt+=("-k ${OPTARG}")
                ;;
            *)
                create_dataset_usage
                ;;
        esac
    done
    shift $((OPTIND-1))
    
    local out_exists=`renku dataset ls | awk -v var=$name '$2==var' | wc -l`
    
    if [ $out_exists -eq 0 ]  
    then
        echo "Creating $name"
        renku dataset create "${name}" "${create_opt[@]}"
    fi
   }


function add_files_to_dataset_by_name () {
   
    local dataset_name=$1
    local file_names=(${@:2})
   
    for file in ${file_names[@]}
    do
        local file_exists=`renku dataset ls-files $dataset_name -I "${file}" | wc -l`
        
        if [ $file_exists -gt 2 ]
        then
            # update file using renku command.
            renku dataset update -I $file $dataset_name
            #renku dataset add -o $dataset_name $file
        else
            echo "Adding $file to the dataset $dataset_name"
            renku dataset add ${dataset_name} ${file}
        fi
    done
   }


## ---------------------------------------------------------------------------------------- ##
## ------------------------ Functions to manage datasets(-files) ---------------------------##
## ---------------------------------------------------------------------------------------- ##

function list_data_by_file_name () {

    # input: file name as specified in config.sh
    # output: all data names with a file of input file name 
    
    local file_name=$1
   
    find_unique `ls ${IN_PATH[$file_name]} | grep ${IN_PREFIX[$file_name]}` | sed "s/${IN_PREFIX[$file_name]}_//" | sed "s/.${IN_EXT[$file_name]}//"
   }




function test_import () {
    echo "WORKING N29041039"
}