#!/usr/bin/env bash

##############################################################################################
#                          Provenance Utils Functions                                        #
#                                                                                            #
# Functions to facilitate provenance and related Knowledge graph related actions with renku. #
# Most functions are generalizable, but some specifically target omnibenchmark scripts and   #
# are only valid or useful in an omnibenchmark context/repository                            #
#                                                                                            #
# Change History                                                                             #
# 20/04/2021  Oksana Riba Grognuz    Original code: functions to interact with provenance    #
#                                    graph and query renku's knowledge base                  #
# 20/05/2021  Almut Lütge            Major update: Update functions to work with new renku   #
#                                    onthology in renku 0.15.1                               #
# 10/06/2021  Almut Lütge            Major update: Update Documentation and add helper       #
#                                    functions.                                              #
#                                                                                            #
#                                                                                            #
# authors: Oksana Riba Grognuz & Almut Lütge                                                 #
##############################################################################################
##############################################################################################
##############################################################################################

##############################################################################################

## --------------------------------------------------------------------------------------- ##
## -------------------------------- Prerequisites  ----------------------------------------##
## --------------------------------------------------------------------------------------- ##


# { # try
#     source src/config.sh
# } || { 
#     echo "Warning, src/config.sh does not exist and may be required for some functions."
# }


##############################################################################################

## ---------------------------------------------------------------------------------------- ##
## ------------------------------- Help Documentations  ------------------------------------##
## ---------------------------------------------------------------------------------------- ##

help_list_dataset_files()
{
   echo "Function to list all dataset files in the current repository, independent of the dataset."
   echo
   echo "Syntax: help_list_dataset_files"
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_list_plan_inputs()
{
   echo "Function to list all existing workflow inputs containing a specific PATTERN."
   echo
   echo "Syntax: list_plan_inputs PROV_JSON PATTERN"
   echo
   echo "Arguments:"
   echo "PROV_JSON      Json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate."
   echo "PATTERN        Pattern to filter plan inputs for. Default is data."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_list_plan_outputs()
{
   echo "Function to list all existing workflow outputs containing a specific PATTERN."
   echo
   echo "Syntax: list_plan_outputs PROV_JSON PATTERN"
   echo
   echo "Arguments:"
   echo "PROV_JSON      Json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate."
   echo "PATTERN        Pattern to filter plan outputs for. Default is data."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_find_files_to_process()
{
   echo "Function to list files that are defined as input in src/config.sh and part of a dataset that is associated to the current project,"
   echo "but not yet input of a workflow."
   echo
   echo "Syntax: find_files_to_process PROV_JSON"
   echo
   echo "Arguments:"
   echo "PROV_JSON      Json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_find_datasets_to_process()
{
   echo "Function to list datasets associated to the current project with an input defined in src/config.sh, but not yet input of a workflow."
   echo
   echo "Syntax: find_datasets_to_process PROV_JSON"
   echo
   echo "Arguments:"
   echo "PROV_JSON      Json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


help_run_command_on_unprocessed()
{
   echo "Function to run a command on all unprocessed files. The command needs to be valid for all unprocessed files. See find_files_to_process -h."
   echo
   echo "Syntax: run_command_on_unprocessed COMMAND PROV_JSON"
   echo
   echo "Arguments:"
   echo "COMMAND        Command to run on unprocessed files. Needs to be valid for all unprocessed input files."
   echo "PROV_JSON      Json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}

get_input_file_path()
{
   echo "Function to get the path of the file with specified input index and dataset."
   echo
   echo "Syntax: get_input_file_pat DATASET_NAME INPUT_INDEX"
   echo
   echo "Arguments:"
   echo "DATASET_NAME   Name of the dataset that the file of interest belongs to."
   echo "INPUT_INDEX    Index name of the file of interest. Needs to be a valid key of IN_PREFIX (usually specified in src/config.sh)."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}

get_input_file_path()
{
   echo "Function to get the path of the file with specified input index and dataset."
   echo
   echo "Syntax: get_input_file_pat DATASET_NAME INPUT_INDEX"
   echo
   echo "Arguments:"
   echo "DATASET_NAME   Name of the dataset that the file of interest belongs to."
   echo "INPUT_INDEX    Index name of the file of interest. Needs to be a valid key of IN_PREFIX (usually specified in src/config.sh)."
   echo
   echo "Options:"
   echo "h              Print this Help."
   echo
}


##############################################################################################

## ---------------------------------------------------------------------------------------- ##
## ------------------------------- Helper functions/snippets -------------------------------##
## ---------------------------------------------------------------------------------------- ##

function find_unique () {
    echo "$@" | tr ' ' '\n' | sort | uniq -u
}

##############################################################################################

## ---------------------------------------------------------------------------------------- ##
## ---------------------- Functions to query KG using access points ------------------------##
## ---------------------------------------------------------------------------------------- ##

function list_dataset_files () {
    
    # Output: List of all datasets files in the current project.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_list_dataset_files
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    # List all dataset files
    local prop="http://schema.org/hasPart"
    files=(`renku dataset ls --format json-ld | jq -r '.[] | .[] | .'\"$prop\"'? | select( . != null) | .[]."@id"'`)
    
    for one_file in ${files[@]}
    do
        echo data/${one_file##https://renkulab.io/blob/*/data/}
    done
    
    }


function get_input_file_path(){

    # Input1: Dataset name of the file to get the path
    # Input2: Index name of the file to get the path 
    # Output: Path of the input file with the specified index and dataset.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_get_input_file_path
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    local data_name=$1
    local input_index=$2
    echo data/${data_name}/${IN_PREFIX[$input_index]}_*.${IN_EXT[$input_index]}
}

## ---------------------------------------------------------------------------------------- ##
## -------------------- Functions to query KG using provenance.json ------------------------##
## ---------------------------------------------------------------------------------------- ##


function list_plan_inputs () {

    # Input1: json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate.
    # Input2: Pattern to filter plan inputs for. Default "data". 
    # Output: All files of pattern $2(default data) listed as inputs for any existing workflow
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_list_plan_inputs
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    # find pland inputs
    local json_file=${1:-.renku/provenance.json}
    local pattern=${2:-data}
    local prop="https://renkulab.io/plans/"
    local prop2="http://schema.org/defaultValue"
    
    cat $json_file | jq -r '.[] | select(.'\"@id\"' | contains('\"$prop\"') and contains("inputs")) | .'\"$prop2\"'? | .[]."@value" | select(. | contains('\"$pattern\"'))'
}


function list_plan_outputs () {
    
    # Input1: json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate.
    # Input2: Pattern to filter plan outputs for. Default "data". 
    # Output: All files of pattern $2(default data) listed as outputs for any existing workflow
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_list_plan_outputs
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    #find plan outputs
    local json_file=${1:-.renku/provenance.json}
    local pattern=${2:-data}
    local prop="https://renkulab.io/plans/"
    local prop2="http://schema.org/defaultValue"
    
    cat $json_file | jq -r '.[] | select(.'\"@id\"' | contains('\"$prop\"') and contains("outputs")) | .'\"$prop2\"'? | .[]."@value" | select(. | contains('\"$pattern\"'))'
}



function find_files_to_process(){
    
    # Input: json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate.
    # Output: All files that are defined as input in src/config.sh and part of a dataset that is associated to the current project, but not yet input of a workflow. 
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_find_files_to_process
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    # Find files to process
    local json_file=${1:-.renku/provenance.json}
    local pattern=`echo ${IN_PREFIX[@]} | tr ' ' '|'`
    local pattern_excl=`echo $(list_plan_outputs $json_file $OUT_PATH)`
    
    unprocessed_in=$(find_unique `list_dataset_files $json_file` `list_plan_inputs $json_file` \
    | egrep $pattern)
    find_unique $pattern_excl `find_unique $unprocessed_in $pattern_excl`
}



function find_datasets_to_process(){
    
    # Input: json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate.
    # Output: All datasets associated to the current project with an input defined in src/config.sh, but not yet input of a workflow. 
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_find_datasets_to_process
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    # Find datasets to process
    local json_file=${1:-.renku/provenance.json}
    find_files_to_process $json_file | cut -f2 -d '/' | sort | uniq
}


function run_command_on_unprocessed(){
    
    # Input1: Command to run on unprocessed files. Needs to be valid for all unprocessed input files.
    # Input2: json file with the provenance graph stored. Default .renku/provenance.json, which is generated by renku graph generate.
    # Output: Command output for all unprocessed files. See find_files_to_process -h.
    
    # Display help message
    while getopts ":h" option; do
        local OPTIND option h
        case $option in
            h) 
                help_run_command_on_unprocessed
                return 1;;
            \?) 
                echo "Error: Invalid option"
                return 1;;
            esac
    done
    shift $((OPTIND-1))
    
    # Run command
    local command=$1
    local json_file=${2:-.renku/provenance.json}
    files=(`find_files_to_process $json_file`)
    for one_file in ${files[@]}
    do
        $command $one_file
    done
}

##############################################################################################

